#extension GL_OES_EGL_image_external : require
#define SAMPLES 5
precision highp float;
uniform samplerExternalOES uTexture;

varying vec2 vTextureCoord;
varying vec2 vBlurTextureCoord[SAMPLES];

uniform mediump float bilateralFilter;
uniform mediump float contrast;
uniform mediump float whiten;

vec4 skinProcess(vec4 color)
{
    if(bilateralFilter == 0.0)
    {
        return color;
    }
    lowp vec4 centralColor;
    lowp float gaussianWeightTotal;
    lowp vec4 sum;
    lowp vec4 sampleColor;
    lowp float distanceFromCentralColor;
    lowp float gaussianWeight;

    centralColor = texture2D(uTexture, vBlurTextureCoord[2]);
    gaussianWeightTotal = 0.18;
    sum = centralColor * 0.18;

    sampleColor = texture2D(uTexture, vBlurTextureCoord[0]);
    distanceFromCentralColor = min(distance(centralColor, sampleColor) * (10.0 - bilateralFilter/20.0), 1.0);
    gaussianWeight = 0.05 * (1.0 - distanceFromCentralColor);
    gaussianWeightTotal += gaussianWeight;
    sum += sampleColor * gaussianWeight;

    sampleColor = texture2D(uTexture, vBlurTextureCoord[1]);
    distanceFromCentralColor = min(distance(centralColor, sampleColor) * (10.0 - bilateralFilter/20.0), 1.0);
    gaussianWeight = 0.10 * (1.0 - distanceFromCentralColor);
    gaussianWeightTotal += gaussianWeight;
    sum += sampleColor * gaussianWeight;

    sampleColor = texture2D(uTexture, vBlurTextureCoord[3]);
    distanceFromCentralColor = min(distance(centralColor, sampleColor) * (10.0 - bilateralFilter/20.0), 1.0);
    gaussianWeight = 0.10 * (1.0 - distanceFromCentralColor);
    gaussianWeightTotal += gaussianWeight;
    sum += sampleColor * gaussianWeight;

    sampleColor = texture2D(uTexture, vBlurTextureCoord[4]);
    distanceFromCentralColor = min(distance(centralColor, sampleColor) * (10.0 - bilateralFilter/20.0), 1.0);
    gaussianWeight = 0.05 * (1.0 - distanceFromCentralColor);
    gaussianWeightTotal += gaussianWeight;
    sum += sampleColor * gaussianWeight;

    vec4 skinColor = sum / gaussianWeightTotal;

    return skinColor;
}

vec4 whitenProcess(vec4 color)
{
    if(whiten == 0.0)
    {
        return color;
    }
    vec4 whitenColor;
    whitenColor.r = min(color.r + whiten/255.0, 1.0);
    whitenColor.g = min(color.g + whiten/255.0, 1.0);
    whitenColor.b = min(color.b + whiten/255.0, 1.0);
    whitenColor.w = 1.0;
    return whitenColor;
}

vec4 whitenWellProcess(vec4 color)
{
    lowp float nWhiten_;
    nWhiten_ = (whiten - 2.0)/7.0 + 2.0;
    float nR = color.r;
    float nG = color.g;
    float nB = color.b;

    float dstR = (log(nR*(nWhiten_ - 1.0) + 1.0) / log(nWhiten_));
    float dstG = (log(nG*(nWhiten_ - 1.0) + 1.0) / log(nWhiten_));
    float dstB = (log(nB*(nWhiten_ - 1.0) + 1.0) / log(nWhiten_));
    return vec4(dstR, dstG, dstB, 1.0);
}

vec4 contrastProcess(vec4 color)
{
    if(contrast == 0.0)
    {
        return color;
    }
    vec4 contrastColor = vec4(((color.rgb - vec3(0.5)) * contrast / 25.0 + vec3(0.5)), color.w);
    return contrastColor;
}

void main() {
    vec4 base = texture2D(uTexture, vTextureCoord);
    vec4 processColor = skinProcess(base);
    processColor = whitenWellProcess(processColor);
    processColor = contrastProcess(processColor);
    processColor.r = processColor.r + 3.0/255.0;
    gl_FragColor = processColor;
}
